/* ==== maparray.h ==== */
/*  Part of the source of the program "Porrog", a roguelike game.
    Copyright (C) 2002-2003  Tom Barnes-Lawrence
    Current web address: www.angelfire.com/super2/duologue/
    Current email address: tomble@usermail.com
    Postal address: mind your own business

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* This header used to contain the definitions for structures used
   in maparray.c, but is now just function prototypes.

   Should these items be defined here or in structures.h?*/
#define MAPWID 140
#define MAPHGT 140

extern foogrid *map;

/* I'm now going to put in some *more* structures, but ones purely
   pertaining to map generation (well, at the moment, anyway) */

struct area_t {
   int Firstroom; /* Because we'll only store a bounded number of
      rooms, which we'll stick in an array. */
   struct tunnellist_t *Firsttunnel;
   int Size; /* Number of rooms in the area (starts as 1)*/
   struct area_t *Nextarea;/*Because we want a list of em */
};
typedef struct area_t area_t;

struct tunnellist_t {
   int Xstart,Ystart;
   int Xend,Yend;
   area_t *Area;
   struct tunnellist_t *Nexttunnel;
};
typedef struct tunnellist_t tunnellist_t;

typedef enum {
   FACE_N,
   FACE_E,
   FACE_S,
   FACE_W
} direction_t; /* Should be defined elsewhere? */

struct entrance_t {
   int Xcoord,Ycoord;
   direction_t Outfacing; /* the direction you face if you face this
        door from inside the room XP */
   struct entrance_t *Nextentrance;
};
typedef struct entrance_t entrance_t;

typedef struct {
   int Xcentre,Ycentre; /*Hey, seems we *can* say it this way.*/
   int Xstart,Ystart; /*Just so we don't have to keep calculating it*/
   int Width, Height;
   entrance_t *Entrancelist;
   area_t *Area;
   int Nextroom;
} roomdata_t;

void make_map();
void merge_areas(area_t *area1,area_t *area2);
/* MUST REMEMBER that this has to CORRECTLY update the area lists!!!*/

int pick_room(int totalrooms);/* tries to pick a room at random (using
   various methods) that is an actual created room. If it continually
   fails to, it just picks the earliest one in the list*/

int pick_room_from_area(area_t *area); /*As above, but only rooms from
some given area*/

void attempt_joining(int source, int target);
/* source and target are indices to rooms. Has a pop at finding a path
   for a tunnel between them both. If it hits something not yet accessible
   from source, it just builds a tunnel to there instead (and exits).
   If it hits a *room* already accessible from source, then that room
   becomes the new source, and we try again (so we will be more likely
   to get a successful connection).*/

/*
int join_tunnels(area_t *source,int x, int y, direction_t dir);
HAHA! The saviour! Start at point x,y. Move in direction dir.
  If we hit somewhere we can't tunnel into (corner, permanent wall),
  return -1.
  If we hit a tunnel: If it's *ours*, do join_tunnels(source,foundx,foundy,
  dir);.
  if *not* ours, do merge_areas(find_area(foundx,foundy),source);,
  and build_tunnel(source x,y,foundx,foundy);.
  Hold it, no. I'll merge this into attempt_join().*/

int classify_rpos(int source,int target);
/*Classifies relative position of target from source.
  Used by attempt_join()*/

int probe_path(int x,int y, direction_t direction);
/* Starts at x,y, moves in direction. When it hits something that isn't
   PTL_ROCK, it returns tha point at which it finds it.
   If direction is east or west, the return value is an X coord.
   otherwise, the return value is a Y coord. */

void build_tunnel(area_t *area,int xstart,int ystart,int xend,int yend);
/* Tunnels out an area between start and end. Doesn't check where it's
  going (you should have done that already!). Then creates a tunnellist_t
  structure, and sticks it into area. DOESNT check whether to join areas!!*/

int find_room(int x,int y);
 /* As find_area, but expects to find a room. If it doesn't find a room,
   returns -1. If it does, returns index of that room.
   Important for driving through an area on course towards a target room.
   If we hit an already accessible room, we try from there instead (as it
   will obviously be closer). If we hit a not-accessible room, we try
   to connect *to* there. But we can't connect to the point we found the
   room at, which might well be the corner of the room (not good).*/
area_t *find_area(int x,int y);
 /*Takes a map coord, works out what it is, then
  if a room, scans all rooms to find it, else if a tunnel, scans all
  tunnel sections of all areas to find it!!! Probably slow.*/

area_t *choose_small_area(); /*Searches through whole area list, to find
  smallest area. If finds an area of size 1, is likely to just accept it,
  else will give minimum at end. */

/* Now part of ui.h
void draw_map(int cornerx,int cornery);
*/
