/* ==== maparray.c ==== */
/*  Part of the source of the program "Porrog", a roguelike game.
    Copyright (C) 2002-2003  Tom Barnes-Lawrence
    Current web address: www.angelfire.com/super2/duologue/
    Current email address: tomble@usermail.com
    Postal address: mind your own business

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdlib.h>
/* #include <ncurses.h> Should be removed when debugging syms removed*/
#include "porrog_defs.h"
#include "C-foogrid.h"
#include "memutils.h"
#include "structures.h"
#include "maparray.h" /*We need this, but maybe just for the #defines*/
#include "item.h"
#include "crash.h"
#include "dXrng.h"

foogrid *map;
foogrid *tmap;
itemtype_t markerobs[16]; /*Just for testing. */
roomdata_t room[217]; /* Unfortunately now global X( */
area_t *firstarea=NULL;/* Pointer to the first area of list*/

void make_map()
{
   int x,y,dx,dy,mr,mxr,myr,n,flag,s,done=0,tries,totalrooms,target;
   dXrng_roller *d12;
   dXrng_roller *d20;
   tile_t thistile; /* used in tests */
   maploc_t * elem;
   maploc_t *mapstart;
   /* We should have already called initialise_dXrng by now */
   d12=new_dX(12);
   d20=new_dX(20);
   for(n=0;n<10;n++)
      new_itemtype(markerobs+n,48+n,"A marker number");
   for(n=10;n<16;n++)
      new_itemtype(markerobs+n,55+n,"A marker letter");
   map=create_foogrid(MAPHGT,MAPWID,sizeof(maploc_t));
   tmap=create_foogrid(MAPHGT,MAPWID,sizeof(tunnellist_t*)); /*Pointers to em*/
   /* Now we populate that grid */
   mapstart=map->Data;
   for(x=0,elem=mapstart+foogrid_idx(map,0,0); x<(MAPWID-1); elem++,x++)
   {
      elem->Tiletype=PTL_PWALL; /* Can't be passed or mined */
      elem->Mptr=NULL;
      elem->Seen=0;
      elem->Itemlist=NULL;
      elem->Dest=NULL;
   }
   for(x=0,elem=mapstart+foogrid_idx(map,(MAPHGT-1),0); x<(MAPWID-1); elem++,x++)
   {
      elem->Tiletype=PTL_PWALL;
      elem->Mptr=NULL;
      elem->Seen=0;
      elem->Itemlist=NULL;
      elem->Dest=NULL;
   }
   for(y=1;y<MAPHGT-2;y++)
   {
      for(x=0,elem=mapstart+foogrid_idx(map,y,0);x<(MAPWID-1);elem++,x++)
      {
         if(x==0||x==(MAPWID-1)) elem->Tiletype=PTL_PWALL; else elem->Tiletype=PTL_ROCK;
         elem->Mptr=NULL;
         elem->Seen=0;
         elem->Itemlist=NULL;
         elem->Dest=NULL;
      }
   }
   (mapstart+foogrid_idx(map,MAPHGT/2,MAPWID/2))->Tiletype=PTL_RMID;
   room[216].Xcentre=MAPWID/2;
   room[216].Ycentre=MAPHGT/2;
   room[216].Area=(area_t *)counting_malloc(sizeof(area_t));
     /* Make sure there's a room in the middle (start position) */
   room[216].Area->Firstroom=216;
   room[216].Area->Firsttunnel=NULL;
   room[216].Area->Size=1;
   room[216].Area->Nextarea=NULL; /*End of area list. */
   firstarea=room[216].Area;
   totalrooms=1;

   /* Now we make a load more! */
   for(n=0;n<216;n++)
   {
      /* Pick some random spot, then check the squares +-6 squares around it
         for other spots that have already been marked. If we find any
         in *that* area, we don't use the spot we have, but look for another*/
      room[n].Xcentre=-1;room[n].Ycentre=-1; /* If we can't alloc one */
      room[n].Area=NULL;
      for(s=14;s>7;s--)
      {
        /* was using 8+roll_dX(d20)++roll_dX(d8)*/
        /* changed 8 and d8 to 6 and d6, with shrink to 140X map.*/
        x=10+( (6*roll_dX(d20))+(roll_dX(d12)%6) );
        y=10+( (6*roll_dX(d20))+(roll_dX(d12)%6) );
        flag=0;
        for(dy=y-s;dy<=y+s;dy++)
           for(dx=x-s;dx<=x+s;dx++)
           {
              if(dx<0||dx>MAPWID-1||dy<0||dy>MAPHGT-1) continue;
               /* this doesn't make a room here illegal*/
              if( (mapstart+foogrid_idx(map,dy,dx))->Tiletype!=PTL_ROCK)
                 /* We can't build a room around here. */ flag=1;
           }
        if (flag==0)
        {
           (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_RMID;
           room[n].Xcentre=x;room[n].Ycentre=y;
           room[n].Area=(area_t *)counting_malloc(sizeof(area_t));
           room[n].Area->Firstroom=n;
           room[n].Area->Firsttunnel=NULL;
           room[n].Area->Size=1;
     /*    room[n].Area->Nextarea=room[216].Area->Nextarea;
           room[216].Area->Nextarea=room[n].Area;*/
           room[n].Area->Nextarea=firstarea;
           firstarea=room[n].Area;
           /*Attach this area to head
             of area list. Well, OK, 216's area is the head of the list.
             So this goes *behind* the head. Meh.*/
           totalrooms++;
           break; /* Don't look for smaller spaces now. */
        }
      }
   }

   /* Room midpoints established. Find their dimensions!*/
   for(n=0;n<217;n++)
   {
      if (-1==room[n].Xcentre) continue;
/*      add_item(map,y,x,markerobs+1); */
      for(s=14;s>7;s--) /* s of 6 would mean no space between walls*/
      {
         flag=0;
         for(dy=-s;dy<=s;dy++)
            for(dx=-s;dx<=s;dx++)
            {
               x=room[n].Xcentre+dx;y=room[n].Ycentre+dy;
               if(x<0||x>MAPWID-1||y<0||y>MAPHGT-1||(dx==0&&dy==0)) continue;
               if( (mapstart+foogrid_idx(map,y,x))->Tiletype==PTL_RMID)
                   flag=1; /*We've encountered the middle of another room*/
            }
         if (flag==0) break; /*We have a room up to size s that's clear*/
      }
      /* So we now know the maximum size, but we have to check for
         other excavations, to reduce the maximum x and y size */
      mr=(4*s)/7; /* max possible radius */
      /* if s-mr<4 mr=s-4 */
      mr=((s-mr<5)?s-5:mr);
      add_item(map,room[n].Ycentre,room[n].Xcentre,markerobs+s);
      add_item(map,room[n].Ycentre,room[n].Xcentre+1,markerobs+mr);
      do {
        flag=0;
        myr=((roll_dX(d12)*(mr-2))/17)+5; /* We say 5 to leave extra wall space*/
        /* rather than dividing by 11, we divide by 17 to give
           an aspect ratio that looks more square- in fact, more
           horizontal than square!*/
        mxr=((roll_dX(d12)*(mr-2))/11)+5; /* We take 3 off if it's safe*/
        /*myr=(mrmax of 2/3 s, min of 2) */
        /* mxr=(max of 2/3 s, min of 2) */
        /* radius of 2 gives room size (internal) of 5,  wall size of 7*/
        for(dy=-myr;dy<=myr;dy++)
           for(dx=-mxr;dx<=mxr;dx++)
           {
              x=room[n].Xcentre+dx;y=room[n].Ycentre+dy;
              if(x<1||x>(MAPWID-2)||y<1||y>(MAPHGT-2)) {flag=1; break;}
              if(dx==0&&dy==0) continue; /*The middle of the room */
              thistile=(mapstart+foogrid_idx(map,y,x))->Tiletype;
              if( thistile!=PTL_ROCK&&thistile!=PTL_HWALL&&thistile!=PTL_VWALL&&thistile!=PTL_CORNER)
              /* The perimeter walls *used* to be the same type as the rock*/
              {
                  flag=1;
                  break;
              }
           }
      }while(flag==1); /* we now have suitable dimensions! */
      myr=myr-3;mxr=mxr-3; /* Remove the wall compensation to draw it. */
      /*Proceed to dig out the room */
      for(dy=-myr;dy<=myr;dy++)
         for(dx=-mxr;dx<=mxr;dx++)
         {
            x=room[n].Xcentre+dx;y=room[n].Ycentre+dy;
            if((mapstart+foogrid_idx(map,y,x))->Tiletype!=PTL_RMID) (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_FLOOR;
         }
      /* Hollowed out?*/
      room[n].Width=3+(mxr*2); /* mxr is horizontal radius, not width,
         and doesn't count the centre, nor the perimeter walls. */
      room[n].Height=3+(myr*2); /*Likewise*/
      room[n].Xstart=room[n].Xcentre-(mxr+1);
      room[n].Ystart=room[n].Ycentre-(myr+1);
      /* build the perimeter walls */
      dy=(myr+1);
      for(dx=-mxr;dx<=mxr;dx++)
      {
         x=room[n].Xcentre+dx;y=room[n].Ycentre-dy;
         (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_HWALL;
         y=room[n].Ycentre+dy;
         (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_HWALL;
      }
      dx=(mxr+1);

      x=room[n].Xcentre-dx;y=room[n].Ycentre-dy;
      (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_CORNER;

      x=room[n].Xcentre+dx;
      (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_CORNER;

      y=room[n].Ycentre+dy;
      (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_CORNER;

      x=room[n].Xcentre-dx;
      (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_CORNER;

      for(dy=-myr;dy<=myr;dy++)
      {
         x=room[n].Xcentre-dx;y=room[n].Ycentre+dy;
         (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_VWALL;
         x=room[n].Xcentre+dx;
         (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_VWALL;
      }

      room[n].Nextroom=-1;
      /* Now we'd do stuff about links to other rooms. */

   }
   while(done==0)
   {
       n=pick_room(totalrooms);
       for(tries=0;tries<5;tries++)
       {
          target=pick_room(totalrooms);
          if (target!=n&&(abs(room[n].Xcentre-room[target].Xcentre)<35)&&(abs(room[n].Ycentre-room[target].Ycentre)<35)&&(room[n].Area!=room[target].Area))
              /*EUREKA! We have one. Somehow. Indicate this fact*/
              tries=6; /*Do we still need to break here?*/
       }
       if(7==tries) /* perhaps this should be 7 if above is 6?? */
       {
          /* This means we have randomly found 2 rooms that
             are suitable for joining.*/
          if(room[n].Xcentre==-1) printf("Source room doesnt exist!");
          if(room[target].Xcentre==-1) printf("Target room doesnt exist!");
          attempt_joining(n,target); /*we won't *necessarily* do so!*/
          /* if path blocked by something not yet accessible from n,
            try to connect to that instead!*/
          /* else path must be blocked by something already accessible
          from n, so we try to steamroller past any tunnels we find
          to see if we can it a room accessible from n. Then we try
          to see if the path between that room and target it blocked.*/
       } else {
          /*We obviously failed to find a suitable target. Take
            a different approach. */
          /* find the smallest area. Then pick a room from that
          area that can make a tunnel to somewhere it cant yet
          access. Each time it hits a room already connected,
          restart the attempt from that room.*/
          n=pick_room_from_area(choose_small_area());
          for(tries=0;tries<15;tries++)/* Should be enough?*/
          {
             target=pick_room(totalrooms);
             if (target!=n && room[n].Area!=room[target].Area)
             /* We don't care where it is, if it's too far we'll
               connect to somewhere else anyway*/
                tries=19;
          }
          if (15==tries) /*We couldn't find one?? Search harder.*/
          {
             for(tries=0;tries<217;tries++)/*tries=roomno*/
             {
                if(tries!=n &&room[tries].Xcentre!=-1&&room[n].Area!=room[tries].Area)
                {
                   target=tries;
                   break;
                }
             }
             if(room[tries].Xcentre==-1)
             { addstr("Picked nonexistent room!");crash(NO_MEM);}
             if (217==tries) crash(INCONSISTENT); /* We couldnt find one! */
          }
          if(target==-1) {addstr("make_map:target=-1!");crash(NO_MEM);}
          attempt_joining(n,target);
       }

       /* If we find the first area we can find has a size of
         totalrooms, then everything is connected, so done=1;
         NB: room 216 is GUARANTEED to exist (room 0 is not!!!)
         so might as well test room[216].Area->Size for this.*/
/*      if ((room[216].Area->Size)>2) done++;
*/
       if((choose_small_area()->Size)>1){move(0,0);addstr("sml");done=1;}
       if(firstarea->Size==totalrooms){move(1,0);addstr("first");done=1;}
       if(firstarea->Nextarea==NULL) {move(2,0);addstr("one");done=1;}
   }
   /* We should now have everything fully accessible! */
   free_areas();/*Tidy up!*/

}



int pick_room(int totalrooms)
{
    int roomno,count,seen;
    static dXrng_roller *d12=NULL;
    static dXrng_roller *d20=NULL;
    if (NULL==d12){d12=new_dX(12);d20=new_dX(20);}
    count=(totalrooms*((12*roll_dX(d20))+roll_dX(d12))/240);
    /* HOPEFULLY count should get truncated such that count < totalrooms.*/
    /* If not, we will likely get an out of bounds. Check, in case. */
    if (count>=totalrooms) addch("Picked room higher than totalrooms");
    for (roomno=-1,seen=0;seen<=count;roomno++,seen+=(room[roomno].Xcentre!=-1)?1:0);
    if (roomno>216||roomno==-1) addch("Picked impossible room");
    if (room[roomno].Xcentre==-1) addch("Picked unmade room");
    /* free(d12);free(d20); Whoops! Mustn't forget that.*/
    return(roomno);
    /* Kept as a reminder of how I'm liable to program whilst too tired
    for (tries=0,roomno=218;tries<10&&roomno>216;tries++)
        roomno=(12*roll_dX(d20))+roll_dX(d12);
    if (roomno>216||room[roomno].Xcentre==-1)
    { that was no good, lets try a different way
       for (tries=0,count=0;tries<1000 && count>=totalrooms;tries++)
           count=(12*roll_dX(d20))+roll_dX(d12);
       if (count>=totalrooms) count=0;  Ah fudge it 
       for (roomno=0,seen=0;seen<count;roomno++)
           seen+=(room[roomno].Xcentre!=-1)?1:0;  Just to be sure...
       roomno should now point to the count_th actual room
       if (roomno>216) crash(OUT_OF_BOUNDS); BIG foobar!!
    }
    return(roomno);
    */
}

int pick_room_from_area(area_t *area)
{
    /* Much the same as pick_room()*/
    int roomno,count,seen,totalrooms=area->Size;
    static dXrng_roller *d12=NULL;
    static dXrng_roller *d20=NULL;
    if (NULL==d12){d12=new_dX(12);d20=new_dX(20);}
    count=(totalrooms*((12*roll_dX(d20))+roll_dX(d12))/240);
    if (count>=totalrooms) crash(OUT_OF_BOUNDS);
    for (roomno=0,seen=0;seen<count;roomno=room[roomno].Nextroom)
        seen+=(room[roomno].Xcentre!=-1)?1:0;
    if (roomno>216||-1==roomno) crash(OUT_OF_BOUNDS);
    /* Presumably if we get a -1, it should crash anyway??*/
    if(room[roomno].Xcentre==-1) crash(OUT_OF_BOUNDS);
    return(roomno);
}

/* 
void attempt_joining(int source, int target)
{
    int found;   any rooms we come across on the way
    direction_t dir;  Direction chosen for tunnels
    int x,y;   temporary vars for starting tunnels
    int xstart,xend,ystart,yend;   Define points for doors
    int px,py;   temporary vars for probing locations
    area_t *sourcearea=room[source].Area;
    area_t *probearea;
    int sx,sy,tx,ty;    Centres of rooms 
    int sw,sh,tw,th;    Dimensions of rooms
    int sx1,sy1,sx2,sy2,tx1,ty1,tx2,ty2;   Boundaries of rooms
    maploc_t *mapstart=map->Data;
    sx=room[source].Xcentre;sy=room[source].Ycentre;
    tx=room[target].Xcentre;ty=room[target].Ycentre;

    sw=room[source].Width;sh=room[source].Height;
    tw=room[target].Width;th=room[target].Height;
    sx1=room[source].Xstart;sx2=sx1+sw-1;
    sy1=room[source].Ystart;sy2=sy1+sh-1;

    tx1=room[target].Xstart;tx2=tx1+tw-1;
    ty1=room[target].Ystart;ty2=ty1+th-1;
       2 rooms must have totaly non-overlapping coordinates
       in AT LEAST one dimension.
       Assuming they're separated in the Y axis:

       +-----+   #        +--+  #     +--+
       |     |   #        |  |  #     |  |
       +-----+   #        +--+  #     +--+
                 #              #
         +-+     # +--+         #   +--+
         | |     # |  |         #   |  |
         +-+     # +--+         #   +--+
      #####################################
         +-+     # +--+         #   +--+
         | |     # |  |         #   |  |
         +-+     # +--+         #   +--+
                 #              #
       +-----+   #        +--+  #     +--+
       |     |   #        |  |  #     |  |
       +-----+   #        +--+  #     +--+
     #######################################
     Are the only possibilities.
     Of course, they could be separated in the X axis
     instead, in which case you would have those
     rotated 90 degrees.
    # If separated by Y, sy2<ty1 || ty2<sy1
    #  If separated by X, sx2<tx1 || tx2<sx1
    if (sy2-1<ty1+1||ty2-1<sy1+1)  separated in Y axis
    {
       if (sx2-1<tx1+1||tx2-1<sx1+1)  separated in X too!
       {
          addch('/');refresh();
          This one the most serious! Tunnels with a corner
       } else {
          addch('|');refresh();
         hey! We can just make 1 vertical tunnel? Unless
          the overlap is too small, which could only happen
          in the *partial* overlap cases, in which case we treat
          as "separated in X"
          #if (sw<tw) x=sx; Because source is narrower
          #else x=tx; That only works if one is within t'other!
          xstart=MAX(sx1,tx1)+1;xend=MIN(sx2,tx2)-1;
          x=(xstart+xend)/2;
          dir=(sy<ty)?FACE_S:FACE_N;
          y=(FACE_N==dir)?sy1:sy2;  WAS FACE_N
          py=probe_path(x,y,dir); This *might* be an untunnelable bit!
          If it's the edge of the map... Hmm, that shouldn't happen here.
          If it's the corner of a room, then if it's in sourcearea,
          that still counts as part of the room.
          If it's the corner of a room *not* in sourcearea, we can't
          do anything with it!!! Can we? 
          All this stuff should be in a further function.
          probearea=find_area(x,py);
          if(probearea==NULL)
          {
             return;
          }
          if (sourcearea!=probearea)
          {
             build_tunnel(sourcearea,x,y,x,py);
             merge_areas(sourcearea,probearea);
             return;
          }
          if ((mapstart+foogrid_idx(map,y,x))->Tiletype==PTL_NSTUNNEL||
             (mapstart+foogrid_idx(map,y,x))->Tiletype==PTL_WETUNNEL)
          { #Ah. Well we can't do anything with that. Keep looking till
            #we hit a room. If that room is one of ours, call
            #attempt_joining(find_room(x,py),target); return;
            #If it is not one of ours, er, Hmm.
          }
       }
    } else {  cant be separated in Y axis, cutting out 2 options
       # yet again, we should hopefully be able to just make
       #  a single horizontal tunnel! Perhaps. If not, we treat
       # as "separated in Y"
       addch('-');refresh();
         # if (sh<th) y=sy;
         #else y=ty;  Should be a ?:;
       ystart=MAX(sy1+1,ty1+1);yend=MIN(sy2-1,ty2-1);
       y=(ystart+yend)/2;
       dir=(sx<tx)?FACE_E:FACE_W;
       x=(FACE_W==dir)?sx1:sx2; WAS FACE_W
       px=probe_path(x,y,dir);
       if(px>MAPWID-2||px<2) crash(OUT_OF_BOUNDS);
       probearea=find_area(px,y);
       if(probearea==NULL)return;
       if (sourcearea!=probearea)
       {
           build_tunnel(sourcearea,x,y,px,y);
           merge_areas(sourcearea,probearea);
           return;
       }
    }
     we want to try doing the "do this but with different source"
       thing when appropriate; for the moment, we'll just recurse.
       Later will change that to iterate (for speed)
}
*/

int probe_path(int x,int y,direction_t direction)
{
    int crs;
    maploc_t *mapstart=map->Data;
    switch(direction)
    {
        case FACE_N:
          for(crs=y-1;crs>1;crs--)
             if ((mapstart+foogrid_idx(map,crs,x))->Tiletype!=PTL_ROCK||(mapstart+foogrid_idx(map,crs,x-1))->Tiletype==PTL_TUNNEL||(mapstart+foogrid_idx(map,crs,x+1))->Tiletype==PTL_TUNNEL)
                break;
          if(crs==y)crash(NO_MEM);
          break;
        case FACE_E:
          for(crs=x+1;crs<MAPWID;crs++)
             if ((mapstart+foogrid_idx(map,y,crs))->Tiletype!=PTL_ROCK||(mapstart+foogrid_idx(map,y-1,crs))->Tiletype==PTL_TUNNEL||(mapstart+foogrid_idx(map,y+1,crs))->Tiletype==PTL_TUNNEL)
                break;
          break;
        case FACE_S:
          for(crs=y+1;crs<MAPHGT;crs++)
             if ((mapstart+foogrid_idx(map,crs,x))->Tiletype!=PTL_ROCK||(mapstart+foogrid_idx(map,crs,x-1))->Tiletype==PTL_TUNNEL||(mapstart+foogrid_idx(map,crs,x+1))->Tiletype==PTL_TUNNEL)
                break;
          break;
        case FACE_W:
          for(crs=x-1;crs>1;crs--)
             if ((mapstart+foogrid_idx(map,y,crs))->Tiletype!=PTL_ROCK||(mapstart+foogrid_idx(map,y-1,crs))->Tiletype==PTL_TUNNEL||(mapstart+foogrid_idx(map,y+1,crs))->Tiletype==PTL_TUNNEL)
                break;
          break;
        default:
          crash(-1); /*Should find a better value!*/
    }
    return(crs);
    /* Hey, that wasn't too painful!*/
    /* DOH!!! Need to check for tunnels either side of path.*/
}

void build_tunnel(area_t *area, int xstart, int ystart, int xend, int yend)
{
    tunnellist_t *thissection;
    int x,y;
    maploc_t *mapstart=map->Data;
    tunnellist_t **tmapstart=tmap->Data; /*Newly added*/
    direction_t dir;
    thissection=(tunnellist_t *)counting_malloc(sizeof(tunnellist_t));
    thissection->Xstart=xstart;thissection->Ystart=ystart;
    thissection->Xend=xend;thissection->Yend=yend;
    /* Ah. This upsets find_area(). which assumes start<=end. Hard to change*/
    if(xstart>xend){thissection->Xstart=xend; thissection->Xend=xstart;}
    if(ystart>yend){thissection->Ystart=yend; thissection->Yend=ystart;}
    thissection->Area=area;
    thissection->Nexttunnel=area->Firsttunnel;area->Firsttunnel=thissection;

    if(xstart!=xend && ystart!=yend)
    {
      printf("\nX: %d Y: %d x: %d y: %d\n",xstart,ystart,xend,yend);
      printf("Not a straight-line tunnel");
      crash(OUT_OF_BOUNDS);
    }
      /* Means that the tunnel is not orthogonal*/
    if (xstart<xend) dir=FACE_E;
    else if (xstart>xend) dir=FACE_W;
    else if (ystart<yend) dir=FACE_S;
    else dir=FACE_N;
    switch(dir)
    {
        case FACE_N:
          x=xstart;
          for(y=ystart;y>=yend;y--)
          {
             (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_TUNNEL;
             *(tmapstart+foogrid_idx(tmap,y,x))=thissection;
          }
          break;
        case FACE_E:
          y=ystart;
          for(x=xstart;x<=xend;x++)
          {
             (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_TUNNEL;
             *(tmapstart+foogrid_idx(tmap,y,x))=thissection;
          }
          break;
        case FACE_S:
          x=xstart;
          for(y=ystart;y<=yend;y++)
          {
             (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_TUNNEL;
             *(tmapstart+foogrid_idx(tmap,y,x))=thissection;
          }
          break;
        case FACE_W:
          y=ystart;
          for(x=xstart;x>=xend;x--)
          {
             (mapstart+foogrid_idx(map,y,x))->Tiletype=PTL_TUNNEL;
             *(tmapstart+foogrid_idx(tmap,y,x))=thissection;
          }
          break;
    }
}

void merge_areas(area_t *area1, area_t *area2)
{
   /* This *REQUIRES* that room[] is GLOBAL. The problem is, that
     means that unless we create it with malloc(), that array is
     going to just sit around the whole time whether we need it or not*/
   int rlist;
   tunnellist_t *tun;
   area_t *next;
   area_t *scan;
   if (area1==area2) return; /*Nothing to do!*/
   if (NULL==area1)
   {
     addstr("area1 is null!");
     refresh();crash(INCONSISTENT);
   }
   if (NULL==area2)
   {
     addstr("area2 is null!");
     refresh();crash(INCONSISTENT);
   }
   for (rlist=area1->Firstroom;room[rlist].Nextroom!=-1;rlist=room[rlist].Nextroom)
      room[rlist].Area=area2;
   /*Now we have the end of list 1.*/
   room[rlist].Area=area2; /*I think we need that again? */
   room[rlist].Nextroom=area2->Firstroom;

   /*I think that mebbe area1 might not have any tunnel??
    Either, way, a segfault happens here!*/
   for (tun=area1->Firsttunnel;tun->Nexttunnel!=NULL;tun=tun->Nexttunnel)
   {
      tun->Area=area2;
   }
   tun->Area=area2; /*again, we repeat that */
   tun->Nexttunnel=area2->Firsttunnel;

   area2->Firstroom=area1->Firstroom;
   area2->Firsttunnel=area1->Firsttunnel;
   area2->Size+=area1->Size;
   /* NOW we want to remove area1 from the area list. OUCH!
     THIS MUST BE DONE!*/
   next=area1->Nextarea; /* The bit we use to replace area1 in the list*/
   if (firstarea==area1)
   { /* The exception to the rule: area1 is head of the list.*/
      firstarea=next; /* That's all folks!*/
   } else {
     for(scan=firstarea;scan->Nextarea!=area1;scan=scan->Nextarea)
     {
        if(scan->Nextarea==NULL)
         {addstr("Found end of area list!");refresh();}
     }
     /*We've now found scan->Nextarea==area1. So we splice here.*/
     scan->Nextarea=next;
   } /*area1 should now be out of the list. Yes?*/
   counting_free(area1); /*we *can* do that, right? */
   show_areas();/* To test*/
   /*Hopefully there should be no pointers to area1 left now.*/
}

area_t *find_area(int x, int y)
{
   area_t *testarea=firstarea;
   tunnellist_t *testtun;
   int r;
   maploc_t *mapstart=map->Data;
   tunnellist_t **tmapstart=tmap->Data;/* Just testing so far*/
   if ((mapstart+foogrid_idx(map,y,x))->Tiletype==PTL_PWALL)
   {
      addstr("Permanent wall found!!");
      crash(OUT_OF_BOUNDS);
   }
   if ((mapstart+foogrid_idx(map,y,x))->Tiletype==PTL_TUNNEL)
   { /* This is a tunnel. Ah. we have to search all tunnel sections..*/
     for(;testarea!=NULL;testarea=testarea->Nextarea)
     {
        if(testarea==NULL) crash(INCONSISTENT);
        for(testtun=testarea->Firsttunnel;testtun!=NULL;testtun=testtun->Nexttunnel)
        {
          if (testtun==NULL) crash(INCONSISTENT);
          if(testtun->Xstart<=x && testtun->Xend>=x &&
            testtun->Ystart<=y && testtun->Yend>=y) /*This'll do*/
             return(testtun->Area); /*Here you go!*/
        }
     }
     addch('X');refresh();
     /*Urp! Didn't find anything. Major foobar!!!*/
     return (NULL);/*temporary*/
     crash(INCONSISTENT); return(NULL); /*To shut up gcc*/
   } else {
    /* Technically it might not be, but we'll assume it's part of a room.*/
    /* This can now be given from room[find_room(x,y)]->Area;
       So we should remove this?*/
     for(r=0;r<217;r++)
     {
         if(room[r].Xcentre!=-1&&
     (room[r].Xstart<=x&&room[r].Xstart+room[r].Width>=x&&
      room[r].Ystart<=y&&room[r].Ystart+room[r].Height>=y))
              return(room[r].Area); /*Yay!*/
     }
     /* Obviously wasn't a room then.*/
     /* It might have been just rock.*/
     return(NULL);
   }
}

int find_room(int x,int y)
{
   int r;
   for (r=0;r<217;r++)
   {
       if(room[r].Xcentre!=-1 &&
       (room[r].Xstart<=x && room[r].Xstart+room[r].Width-1>=x &&
        room[r].Ystart<=y && room[r].Ystart+room[r].Height-1 >=y))
            return(r);
   }
   crash(INCONSISTENT);
   return(-1); /*Not part of any room we can find!*/
}

area_t *choose_small_area()
{
   int minsize=218;   /*Min size found*/
   area_t *carea; /* area with that size*/
   area_t *tarea; /*Area we are testing.*/
   static dXrng_roller *d4=NULL;
   if (NULL==d4) d4=new_dX(4);
   for (tarea=firstarea;tarea!=NULL;tarea=tarea->Nextarea)
   {
      if(tarea->Size==1)
      {
         minsize=1;carea=tarea;
         if (!roll_dX(d4))break; /*  We might stop looking here*/
      }
      if(tarea->Size<minsize)
      {
         minsize=tarea->Size;
         carea=tarea;
      }
   }
   return(carea);
}

void show_areas()
{
    int tr;
    for(tr=0;tr<217;tr++)
    {
       if(room[tr].Xcentre!=-1)/* This is a real room*/
       {
          if(room[tr].Area->Nextarea==firstarea)
          {
             addstr("Room found with Nextarea pointing to 1st area!");
             refresh();
          }
       }
    }
}
void free_tunnels(tunnellist_t *tl)
{
    tunnellist_t *next;
    for(;tl!=NULL;next=tl->Nexttunnel,counting_free(tl),tl=next);
}
void free_areas()
{
    /*There should only be one area, firstarea.*/
    area_t *area;
    area_t *next;
    for(area=firstarea;area!=NULL;next=area->Nextarea,counting_free(area),area=next)
    {
       free_tunnels(area->Firsttunnel);
    }
}
