/* ==== ui.c ==== */
/*  Part of the source of the program "Porrog", a roguelike game.
    Copyright (C) 2002-2003  Tom Barnes-Lawrence
    Current web address: www.angelfire.com/super2/duologue/
    Current email address: tomble@usermail.com
    Postal address: mind your own business

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdlib.h>
#include <ncurses.h>
#include "porrog_defs.h"
#include "C-foogrid.h"
#include "memutils.h"
#include "structures.h"
#include "maparray.h" /*We need this, but maybe just for the #defines*/
#include "item.h"
#include "crash.h"
#include "dXrng.h"

/* Declared as extern in maparray.h?
foogrid *map; */
/* itemtype_t markerobs[16]; Just for testing. */
/* roomdata_t room[217];  Unfortunately now global X( */


void draw_map(int cornerx,int cornery)
{
   /* We assume that curses is already initialised. */
   int x,y,sx,sy,tile;
   maploc_t *elem;
   maploc_t *mapstart=map->Data;
   char sym;
   for (y=cornery,sy=1;sy<23;y++,sy++)
      for (elem=mapstart+foogrid_idx(map,y,cornerx),sx=30;sx<80;elem++,sx++)
      {
          move(sy,sx);
          if(NULL!=elem->Mptr) sym=elem->Mptr->Mobtype->Symbol;
          else if (NULL!=elem->Itemlist) sym=elem->Itemlist->Itemtype->Symbol;
          else{
            tile=elem->Tiletype; /* show the floor */
            switch(tile)
            {
                case PTL_PWALL:
                case PTL_CORNER:
                   sym='*';break;
                case PTL_ROCK:
                   sym='#';break;
                case PTL_HWALL:
                   sym='-';break;
                case PTL_VWALL:
                   sym='|';break;
                case PTL_DOOR:
                   sym='+';break;
                case PTL_TUNNEL:
                   sym=':';break;
                case PTL_FLOOR:
                case PTL_RMID:
                   sym='.';break;
                default:
                   sym=''; /* Foobar!!!*/
            }
          }
          addch(sym);
      }
   refresh();
}

void msg_window(char *msg)
{
   int x,y;
   int length=strlen(msg);
   int xe=43+(length/2);
   for(x=37-(length/2);x<xe;x++)
   {
      move(8,x);addch('-');
      move(12,x);addch('-');
   }
   for(y=9;y<12;y++)
   {
      move(y,37-(length/2));addch('|');
      move(y,xe);addch('|');
   }
   move(8,37-(length/2));addch('+');
   move(12,37-(length/2));addch('+');
   move(8,xe);addch('+');
   move(12,xe);addch('+');
   move(10,40-(length/2));addstr(msg);
   move(14,0);
   refresh();
}

void display_heap(item_t *list)
{
   int count=0,x=0;
   item_t *scan;
   WINDOW *win;
   for(scan=list;scan!=NULL;scan=scan->Next)
      count++;
   if(count==0)return;
   /*
   move(8,1);
   addch('0'+(count/10));
   addch('0'+(count%10));
   refresh(); */
   win=newwin(count+2,60,12-(count/2),10);
   for(count=0,scan=list;scan!=NULL;scan=scan->Next,count++)
   {
     wmove(win,count+1,0);waddch(win,'|');
     waddch(win,scan->Itemtype->Symbol);waddch(win,':');
     waddstr(win,scan->Itemtype->Name);
     wmove(win,count+1,59);waddch(win,'|');
   }
   wmove(win,0,0);waddch(win,'+');
   wmove(win,0,59);waddch(win,'+');
   wmove(win,count+1,0);waddch(win,'+');
   wmove(win,count+1,59);waddch(win,'+');
   for(x=1;x<59;x++)
   {  wmove(win,0,x);waddch(win,'-');
      wmove(win,count+1,x);waddch(win,'-');
   }
   touchwin(win);
   wrefresh(win);
   while(' '!=getch())
   {}
   werase(win);wrefresh(win); /* blank it!*/
   delwin(win);

}
